package com.fusion.fusionsdk.utils;

import android.app.Activity;

import com.touchxd.fusionsdk.FusionAdSDK;
import com.touchxd.fusionsdk.ads.interstitial.InterstitialAd;
import com.touchxd.fusionsdk.ads.interstitial.InterstitialAdListener;
import com.touchxd.fusionsdk.ads.rewardvideo.RewardVideoAd;
import com.touchxd.fusionsdk.ads.rewardvideo.RewardVideoAdListener;
import com.touchxd.fusionsdk.model.AdCode;

import io.flutter.plugin.common.EventChannel;

public class AdUtils {

    public static void loadRewardVideo(final Activity activity, String codeId, String userId, String ext, final EventChannel.EventSink eventSink) {
        AdCode.Builder builder = new AdCode.Builder().setCodeId(codeId);
        if (userId != null) {
            builder.setUserId(userId);
        }

        if (ext != null) {
            builder.setExt(ext);
        }
        AdCode adCode = builder.build();
        FusionAdSDK.loadRewardVideoAd(activity, adCode, new RewardVideoAdListener() {
            @Override
            public void onRewardVideoAdLoad(RewardVideoAd rewardVideoAd) {
                if (eventSink != null)
                    eventSink.success(EventUtils.buildEventMap(Constants.ON_AD_LOAD));
                rewardVideoAd.show(activity);
            }

            @Override
            public void onRewardVideoCached() {
                if (eventSink != null)
                    eventSink.success(EventUtils.buildEventMap(Constants.ON_VIDEO_CACHED));
            }

            @Override
            public void onAdShow() {
                if (eventSink != null)
                    eventSink.success(EventUtils.buildEventMap(Constants.ON_AD_SHOW));
            }

            @Override
            public void onAdClicked() {
                if (eventSink != null)
                    eventSink.success(EventUtils.buildEventMap(Constants.ON_AD_CLICK));
            }

            @Override
            public void onAdClosed() {
                if (eventSink != null) {
                    eventSink.success(EventUtils.buildEventMap(Constants.ON_AD_CLOSE));
                    eventSink.endOfStream();
                }

            }

            @Override
            public void onVideoComplete() {
                if (eventSink != null)
                    eventSink.success(EventUtils.buildEventMap(Constants.ON_VIDEO_COMPLETE));

            }

            @Override
            public void onVideoError() {
                if (eventSink != null)
                    eventSink.success(EventUtils.buildEventMap(Constants.ON_VIDEO_ERROR));

            }

            @Override
            public void onReward(String data) {
                if (eventSink != null)
                    eventSink.success(EventUtils.buildRewardEventMap(data));
            }

            @Override
            public void onVideoVerify() {
                if (eventSink != null)
                    eventSink.success(EventUtils.buildEventMap(Constants.ON_VIDEO_VERIFY));
            }

            @Override
            public void onError(int i, int i1, String s) {
                if (eventSink != null) {
                    eventSink.success(EventUtils.buildEventErrorMap(i1, s));
                    eventSink.endOfStream();
                }
            }
        });

    }

    public static void loadInterstitialAd(Activity activity, String codeId, final EventChannel.EventSink eventSink) {
        AdCode adCode = new AdCode.Builder().setCodeId(codeId).build();
        FusionAdSDK.loadInterstitialAd(activity, adCode, new InterstitialAdListener() {
            @Override
            public void onInterstitialAdLoad(InterstitialAd interstitialAd) {
                if (eventSink != null)
                    eventSink.success(EventUtils.buildEventMap(Constants.ON_AD_LOAD));
                interstitialAd.show(activity);
            }

            @Override
            public void onAdClicked() {
                if (eventSink != null)
                    eventSink.success(EventUtils.buildEventMap(Constants.ON_AD_CLICK));
            }

            @Override
            public void onAdShow() {
                if (eventSink != null)
                    eventSink.success(EventUtils.buildEventMap(Constants.ON_AD_SHOW));
            }

            @Override
            public void onAdClosed() {
                if (eventSink != null) {
                    eventSink.success(EventUtils.buildEventMap(Constants.ON_AD_CLOSE));
                    eventSink.endOfStream();
                }
            }

            @Override
            public void onError(int i, int i1, String s) {
                if (eventSink != null) {
                    eventSink.success(EventUtils.buildEventErrorMap(i1, s));
                    eventSink.endOfStream();
                }
            }
        });

    }

}
