#import "FusionsdkPlugin.h"
#import <AdSDK/WUInterstitialAd.h>
#import <AdSDK/WURewardVideoAd.h>
#import "BannerAdView.h"

@interface FusionsdkPlugin ()<WUInterstitialAdDelegate,WURewardVideoAdDelegate, FlutterStreamHandler>

@property (nonatomic, strong) WUInterstitialAd *interstitialAd;
@property (nonatomic, strong) WURewardVideoAd *rewardedAd;

@property (nonatomic, strong) FlutterResult callback;
@property (nonatomic, weak) NSObject<FlutterPluginRegistrar> *registrar;

@end

@implementation FusionsdkPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"com.fusion.ad/method"
            binaryMessenger:[registrar messenger]];
  FusionsdkPlugin* instance = [[FusionsdkPlugin alloc] init];
    instance.registrar = registrar;
  [registrar addMethodCallDelegate:instance channel:channel];
    
    FusionAdViewFactory *f = [[FusionAdViewFactory alloc] initWithRegistrar:registrar];
        [registrar registerViewFactory:f withId:@"com.fusion.ad/view"];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    // 建立监听
        NSString *eventId = call.arguments[@"eventId"];
        if ([eventId isKindOfClass:[NSNumber class]]) {
            NSString *channel = [NSString stringWithFormat:@"com.fusion.ad/msg_event_%@", eventId];
            FlutterEventChannel *eventChannel = [FlutterEventChannel eventChannelWithName:channel binaryMessenger:[_registrar messenger]];
            [eventChannel setStreamHandler:self];
        }
  if ([@"loadRewardVideo" isEqualToString:call.method]) {
      NSString *codeId = [call.arguments objectForKey: @"codeId"];
      NSString *userId = [call.arguments objectForKey: @"userId"];
      NSString *ext = [call.arguments objectForKey: @"ext"];
      self.rewardedAd = [[WURewardVideoAd alloc] initWithSlotID:codeId userId:userId extra:ext];
      self.rewardedAd.delegate = self;
  } else if ([@"loadInterstitialAd" isEqualToString:call.method]) {
      NSString *codeId = [call.arguments objectForKey: @"codeId"];
      self.interstitialAd = [[WUInterstitialAd alloc]initWithSlotID:codeId adSize:CGSizeMake(300, 450)];
      self.interstitialAd.delegate = self;
  } else if ([@"getPlatformVersion" isEqualToString:call.method]) {
      result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
  } else {
    result(FlutterMethodNotImplemented);
  }
}

- (FlutterError* _Nullable)onListenWithArguments:(NSString *_Nullable)arguments
                                       eventSink:(FlutterEventSink)events {
    NSLog(@"ad plugin -> onListen:%@", arguments);
    if (events) {
        self.callback = events;
    }
    return nil;
}

- (FlutterError *)onCancelWithArguments:(id)arguments {
    NSLog(@"ad plugin -> onCancelListen:%@", arguments);
    return nil;
}

- (void)interstitialAd:(nonnull WUInterstitialAd *)interstitialAd didFailWithError:(NSError * _Nullable)error {
    if (self.callback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onAdError", @"event", [NSNumber numberWithInteger:error.code], @"code", [error.userInfo objectForKey:@"NSLocalizedDescription"], @"msg", nil];
        self.callback(result);
    }
}

- (void)interstitialAdDidClick:(nonnull WUInterstitialAd *)interstitialAd {
    if (self.callback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onAdClick", @"event", nil];
        self.callback(result);
    }
}

- (void)interstitialAdDidClose:(nonnull WUInterstitialAd *)interstitialAd {
    if (self.callback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onAdClose", @"event", nil];
        self.callback(result);
    }
}

- (void)interstitialAdDidLoad:(nonnull WUInterstitialAd *)interstitialAd {
    [interstitialAd showAdFromRootViewController:[[self class] findCurrentShowingViewController]];
    if (self.callback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onAdLoad", @"event", nil];
        self.callback(result);
    }
}

- (void)interstitialAdDidExposed:(nonnull WUInterstitialAd *)interstitialAd {
    if (self.callback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onAdShow", @"event", nil];
        self.callback(result);
    }
}




- (void)rewardVideoAd:(nonnull WURewardVideoAd *)rewardedVideoAd didFailWithError:(nonnull NSError *)error {
    if (self.callback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onAdError", @"event", [NSNumber numberWithInteger:error.code], @"code", [error.userInfo objectForKey:@"NSLocalizedDescription"], @"msg", nil];
        self.callback(result);
    }
}
- (void)rewardVideoAdDidClick:(nonnull WURewardVideoAd *)rewardedVideoAd {
    if (self.callback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onAdClick", @"event", nil];
        self.callback(result);
    }
}
- (void)rewardVideoAdDidClose:(nonnull WURewardVideoAd *)rewardedVideoAd {
    if (self.callback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onAdClose", @"event", nil];
        self.callback(result);
    }
}

- (void)rewardVideoAdDidExposed:(nonnull WURewardVideoAd *)rewardedVideoAd {
    if (self.callback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onAdShow", @"event", nil];
        self.callback(result);
    }
}

- (void)rewardVideoAdDidLoad:(nonnull WURewardVideoAd *)rewardedVideoAd {
    [rewardedVideoAd showAdFromRootViewController:[[self class] findCurrentShowingViewController]];
    if (self.callback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onAdLoad", @"event", nil];
        self.callback(result);
    }
}

- (void)rewardVideoAdDidPlayFinish:(nonnull WURewardVideoAd *)rewardedVideoAd {
    if (self.callback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onVideoComplete", @"event", nil];
        self.callback(result);
    }
}

- (void)rewardVideoAdDidRewardEffective:(nonnull WURewardVideoAd *)rewardedVideoAd {
    if (self.callback) {
        NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:@"onReward", @"event", nil];
        self.callback(result);
    }
}


// 获取当前显示的 UIViewController
+ (UIViewController *)findCurrentShowingViewController {
    //获得当前活动窗口的根视图
    UIViewController *vc = [UIApplication sharedApplication].keyWindow.rootViewController;
    UIViewController *currentShowingVC = [self findCurrentShowingViewControllerFrom:vc];
    return currentShowingVC;
}

+ (UIViewController *)findCurrentShowingViewControllerFrom:(UIViewController *)vc
{
    // 递归方法 Recursive method
    UIViewController *currentShowingVC;
    if ([vc presentedViewController]) {
        // 当前视图是被presented出来的
        UIViewController *nextRootVC = [vc presentedViewController];
        currentShowingVC = [self findCurrentShowingViewControllerFrom:nextRootVC];

    } else if ([vc isKindOfClass:[UITabBarController class]]) {
        // 根视图为UITabBarController
        UIViewController *nextRootVC = [(UITabBarController *)vc selectedViewController];
        currentShowingVC = [self findCurrentShowingViewControllerFrom:nextRootVC];

    } else if ([vc isKindOfClass:[UINavigationController class]]){
        // 根视图为UINavigationController
        UIViewController *nextRootVC = [(UINavigationController *)vc visibleViewController];
        currentShowingVC = [self findCurrentShowingViewControllerFrom:nextRootVC];

    } else {
        // 根视图为非导航类
        currentShowingVC = vc;
    }

    return currentShowingVC;
}

@end
