import 'package:flutter/services.dart';

typedef AdCallback = void Function();
typedef AdErrorCallback = void Function(int code, String message);

class FusionAdSdk {
  static int eventId = 0;
  static MethodChannel _methodChannel =
  new MethodChannel("com.fusion.ad/method");

  static void loadRewardVideoAd(Map map,
      {
        AdCallback onAdLoad,
        AdCallback onAdShow,
        AdCallback onAdClick,
        AdCallback onVideoCached,
        AdCallback onVideoComplete,
        AdCallback onReward,
        AdCallback onVideoVerify,
        AdCallback onAdClose,
        AdCallback onVideoError,
        AdErrorCallback onAdError
      }) {
    _methodChannel.invokeMethod(
        "loadRewardVideo", {"eventId": ++eventId, "codeId": map["codeId"],"userId":map["userId"],"ext":map["ext"]});

    EventChannel eventChannel =
    EventChannel("com.fusion.ad/msg_event_$eventId");
    eventChannel.receiveBroadcastStream().listen((event) {
      switch (event["event"]) {
        case "onAdLoad":
          onAdLoad?.call();
          break;
        case "onAdShow":
          onAdShow?.call();
          break;
        case "onAdClick":
          onAdClick?.call();
          break;
        case "onVideoCached":
          onVideoCached?.call();
          break;
        case "onVideoComplete":
          onVideoComplete?.call();
          break;
        case "onReward":
          onReward?.call();
          break;
        case "onVideoVerify":
          onVideoVerify?.call();
          break;
        case "onAdClose":
          onAdClose?.call();
          break;
        case "onVideoError":
          onVideoError?.call();
          break;
        case "onAdError":
          onAdError?.call(event["code"], event["msg"]);
          break;
      }
    });
  }


  static void loadInterstitialAd(String codeId,
      {
        AdCallback onAdLoad,
        AdCallback onAdShow,
        AdCallback onAdClick,
        AdCallback onAdClose,
        AdErrorCallback onAdError
      }) {
    _methodChannel.invokeMethod(
        "loadInterstitialAd", {"eventId": ++eventId, "codeId": codeId});

    EventChannel eventChannel =
    EventChannel("com.fusion.ad/msg_event_$eventId");
    eventChannel.receiveBroadcastStream().listen((event) {
      switch (event["event"]) {
        case "onAdLoad":
          onAdLoad?.call();
          break;
        case "onAdShow":
          onAdShow?.call();
          break;
        case "onAdClick":
          onAdClick?.call();
          break;
        case "onAdClose":
          onAdClose?.call();
          break;
        case "onAdError":
          onAdError?.call(event["code"], event["msg"]);
          break;
      }
    });
  }
}
